function h = RMD_ColorMap(varargin)
%h = RMD_ColorMap(aImarisApplicationID,aIntensitrBoundary,gamma,aCropColorBar,aTickNum,aTicks,aColorMap)
% Author:  Ren Mengda, 2017.20.28 @ Singapore
% ############### License#################
% Anybody is free to do anything to this script. No warranty.
% ############### License#################
%<CustomTools>
%      <Menu>
%       <Submenu name="XT_RMD">
%        <Item name="RMD_ColorMap" icon="Matlab">
%          <Command>MatlabXT::RMD_ColorMap(%i)</Command>
%        </Item>
%       </Submenu>
%      </Menu>
%      <SurpassTab>
%        <SurpassComponent name="bpVolume">
%          <Item name="RMD_ColorMap" icon="Matlab">
%            <Command>MatlabXT::RMD_ColorMap(%i)</Command>
%          </Item>
%        </SurpassComponent>
%      </SurpassTab>
%    </CustomTools>
if isequal(nargin,1)
    aImarisApplicationID = varargin{1};
    if isa(aImarisApplicationID, 'Imaris.IApplicationPrxHelper')
        vImarisApplication = aImarisApplicationID;
    else
        if  isequal(sum(cellfun(@(x) ~isempty(strfind(x,'ImarisLib.jar')),javaclasspath('-all'))),0)
            javaaddpath ImarisLib.jar
        end
        vImarisLib = ImarisLib;
        if ischar(aImarisApplicationID)
            aImarisApplicationID = round(str2double(aImarisApplicationID));
        end
        vImarisApplication = vImarisLib.GetApplication(aImarisApplicationID);
    end
    
    IndexC = inputdlg('Input the Channel index (zero based) for colormap');
    while 1
        try
            assert(~isempty(IndexC));
            aIndexC = str2num(IndexC{1});
            break
        catch
            IndexC = inputdlg('Warning! Input the Channel index (zero based) for colormap');
        end
    end
    
    aGamma = vImarisApplication.GetDataSet.GetChannelGamma(aIndexC);
    aMax = vImarisApplication.GetDataSet.GetChannelRangeMax(aIndexC);
    aMin = vImarisApplication.GetDataSet.GetChannelRangeMin(aIndexC);
    aColorTable = vImarisApplication.GetDataSet.GetChannelColorTable(aIndexC);
    vRGBA = double(aColorTable.mColorRGB);
    vRed = mod(vRGBA, 256);
    vRGBA = floor(vRGBA / 256);
    vGreen = mod(vRGBA, 256);
    vRGBA = floor(vRGBA / 256);
    vBlue = mod(vRGBA, 256);
    vRGBA = floor(vRGBA / 256);
    aColorMap = double([vRed,vGreen,vBlue])/255;
    
    CropColorBar = inputdlg('Input the [CropColorBarMin,CropColorBarMax] to crop/extend colorbar, OR just go on.');
    while 1
        try
            assert(~isempty(CropColorBar));
            if ~isempty(CropColorBar{1})
                aCropColorBar = eval(CropColorBar{1});
            else
                aCropColorBar = [aMin,aMax];
            end
            break
        catch
            CropColorBar = inputdlg('Warning! Input the [CropColorBar1,CropColorBar2] to crop/extend colorbar, OR just go on.');
        end
    end
    
    TickNum = inputdlg('Input the number of ticks. If customized later, just go on');
    while 1
        try
            assert(~isempty(TickNum));
            aTickNum = str2num(TickNum{1});
            break
        catch
            TickNum = inputdlg('Warning! Input the number of ticks. If customized later, just go on');
        end
    end
    
    if ~isempty(aTickNum)
        h = RMD_ColorMap(-1,[aMin,aMax],aGamma,iCropColorBar,iTickNum,[],aColorMap);
    else
        
        Ticks = inputdlg('Input the [tick1,tick2,...]');
        while 1
            try
                assert(~isempty(Ticks));
                aTicks = eval(Ticks{1});
                break
            catch
                Ticks = inputdlg('Warning! Input the [tick1,tick2,...]');
            end
        end
        h = RMD_ColorMap(-1,[aMin,aMax],aGamma,aCropColorBar,[],aTicks,aColorMap);
    end
    return
end

%% No input
if ~nargin
    aIntensitrBoundary = [132,10732.11];
    gamma = 1.72;
    aCropColorBar = [150,10000];
    % How many enenly spaced Tick?
    aTickNum = [];
    %aTickNumǿյĻ,ԷǿCropColorBaraIntensitrBoundarylinspaceע,С
    if isempty(aTickNum)
        % or nominate the numeric tick positons u like
        aTicks = [100,500,1000,5000,10000,15000];
    end
    aColorMap = [];
else
    aImarisApplicationID = varargin{1};
    aIntensitrBoundary = varargin{2};
    gamma = varargin{3};
    aCropColorBar = varargin{4};
    aTickNum = varargin{5};
    aTicks = varargin{6};
    aColorMap = varargin{7};
end

%% other rare-used parameter
YTickLabels = [];
%ԸλȷTickʾcharαֵvector,ȵĻѭ油
%                 YTickLabels = ['aaaa';'qwee';'edwe';'e   ';'ewf ';'we  ';];
%                 YTickLabels = [100,500,1000,5000,10000,15000];
%,ʵֻ,ֻcharǰĿոԼм䲻еĿոпոֻռ,˿ȥ
%                 YTickLabels = [char(num2str([100,500,1000,5000,10000,15000]')),repmat(' (USD)',6,1)];
%,ʵֻ,ֻǰĿոtrim
%         YTickLabels = strtrim(cellstr([char(num2str([100,500,1000,5000,10000,15000]')),repmat(' (USD)',6,1)]));
%Ҷ,ʵֻ,ֻǰпոӱ,Ҷ
%         YTickLabels = cellstr([char(num2str([100,500,1000,5000,10000,15000]')),repmat(' (USD)',6,1)]);
%         YTickLabels = cellfun(@(x) strrep(x,' ','  '),YTickLabels,'UniformOutput',false);
aTickDirection = 'in';%'out';
aDirection = 'normal';%'reverse';
aYAxisLocation = 'right';% 'left';% 'right' ;

%% generate a image
aSize = [500,900];
aIMG = linspaceNDim(linspace(aIntensitrBoundary(1),aIntensitrBoundary(2),aSize(1))',aIntensitrBoundary(2)*ones(aSize(1),1),aSize(2));
IMG = imshow(aIMG,aIntensitrBoundary);

%% Generate a colormap
%  A colormap, from Imaris 7.4.2 spectrum4096
%  ͷβް,ҵײ"alphaR+ȫalphaB"
if isempty(aColorMap)
    aColorMap = cMapImaris;
end
% Another Colormap,from jet,
% ׶аҲǰalphaRB
% aColorMap = jet(4096);

%% initial show aColorMap
% colormap(aColorMap);%(600:900,:));%ľֲƻͶmapɫ
% colorbar;

%% colorMap gama correction
aColorMapSkewed = NaN(size(aColorMap));
for i = 1 : size(aColorMap,1)
    aLineProjected = ceil(size(aColorMap,1) * (i/size(aColorMap,1))^(1/gamma));
    aColorMapSkewed(i,:) = aColorMap(aLineProjected,:);
end

%%  show aColorMap
colormap(aColorMapSkewed);%(600:900,:));
h = colorbar;

%% crop color bar based on actual intensity number
if ~isempty(aCropColorBar)
    % set(h,'ylim',CropColorBar);
    % or
    h.Limits = aCropColorBar;
end

%% set tick numeric position
if ~isempty(aTickNum)
    if ~isempty(aCropColorBar)
        h.Ticks = linspace(aCropColorBar(1),aCropColorBar(2),aTickNum);
    else
        h.Ticks = linspace(aIntensitrBoundary(1),aIntensitrBoundary(2),aTickNum);
    end
else
    h.Ticks = aTicks';
end

%% tamper 'YTickLabel' in an recursive way.
if ~isempty(YTickLabels)
    h.YTickLabel = YTickLabels;
end

%% tick inside/outside?
h.TickDirection = aTickDirection;

%% upside-down inverse?
h.Direction = aDirection;

%% adjust Y axis L/R position
h.YAxisLocation = aYAxisLocation;

%% delete image
% delete(IMG);
% or
% axesHandlesToChildObjects = findobj(IMG, 'Type', 'image');
% if ~isempty(axesHandlesToChildObjects)
%     delete(axesHandlesToChildObjects);
% end

%% cMapImaris
    function e = cMapImaris
        aPalPath = 'C:\Program Files\Bitplane\Imaris x64 7.4.2\colorTables\more\Spectrum-4096.pal';
        if exist(aPalPath,'file')
            a=csv2cell(aPalPath);
        else
            a = uigetfile('.pal','Select the current .pal file to import from Imaris programme folder');
            a=csv2cell(aPalPath);
        end
        b = cellfun(@(x) strsplit(x,' '),a,'UniformOutput',false);
        c = cellfun(@(x) toTri(x),b,'UniformOutput',false);
        d = [c{:}]';
        e = reshape(d,3,length(d)/3)'/255;
        % figure
        % surf(peaks)
        % colormap(e);
        % colorbar
        % colormapeditor
    end
%% toTri
    function out = toTri(in)
        out = cellfun(@(x) str2num(x),in);
    end
end

% Data=magic(100);
% c=[1 10/3 10 100/3 100 1000/3 1000 10000/3 10000];
% contourf(log(Data(:,:)),log(c));
% colormap(bone);  %Color palate named "bone"
% caxis(log([c(1) c(length(c))]));
% H = colorbar('FontSize',11,'YTick',log(c),'YTickLabel',c);

%  How to align colorbar YTickLabel as middle/left/right?

% imshow('rice.png')
% h = colorbar;
% h.YTickLabel = cellstr([char(num2str([1,5000000,1000,5000,10000,15000]')),repmat(' (USD)',6,1)]);
% The T tick lable is centralized, which is very ugly. I there a way to make it aligned left or right?

% I fount that in colorbar's TickLabels, one space only take up half of space of character, so I solve this problem by :
% imshow('rice.png')
% h = colorbar;
% YTickLabels = cellstr([char(num2str([1,5000000,1000,5000,10000,15000]')),repmat(' (USD)',6,1)]);
% h.YTickLabel =  cellfun(@(x) strrep(x,' ','  '),YTickLabels,'UniformOutput',false);

% There is no option to align, and the only way to tweak is to adjust the input o f the TickLabel.

% To align left:
% imshow('rice.png')
% h = colorbar;
% YTickLabels = strtrim(cellstr([char(num2str([1,5000000,1000,5000,10000,15000]')),repmat(' (USD)',6,1)]));
% h.YTickLabel =  YTickLabels;
